import { Test, TestingModule } from '@nestjs/testing';
import { GroupsController } from './groups.controller';
import { GroupsService } from '../services/groups.service';
import { CreateGroupDto } from '../dto/create-group.dto';
import { UpdateGroupDto } from '../dto/update-group.dto';

describe('GroupsController', () => {
  let controller: GroupsController;
  let service: GroupsService;

  // Mock data
  const mockGroup = {
    id: 'group1',
    name: 'Test Group',
    projectId: 'project1',
    metadata: {},
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockPerson = {
    id: 'person1',
    name: 'Test Person',
    projectId: 'project1',
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  // Mock service
  const mockGroupsService = {
    create: jest.fn(),
    findAll: jest.fn(),
    findByProjectId: jest.fn(),
    findById: jest.fn(),
    update: jest.fn(),
    remove: jest.fn(),
    addPersonToGroup: jest.fn(),
    removePersonFromGroup: jest.fn(),
    getPersonsInGroup: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      controllers: [GroupsController],
      providers: [
        {
          provide: GroupsService,
          useValue: mockGroupsService,
        },
      ],
    }).compile();

    controller = module.get<GroupsController>(GroupsController);
    service = module.get<GroupsService>(GroupsService);
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('create', () => {
    it('should create a new group', async () => {
      const createGroupDto: CreateGroupDto = {
        name: 'Test Group',
        projectId: 'project1',
        metadata: {},
      };

      mockGroupsService.create.mockResolvedValue(mockGroup);

      const result = await controller.create(createGroupDto);

      expect(mockGroupsService.create).toHaveBeenCalledWith(createGroupDto);
      expect(result).toEqual(mockGroup);
    });
  });

  describe('findAll', () => {
    it('should return all groups when no projectId is provided', async () => {
      mockGroupsService.findAll.mockResolvedValue([mockGroup]);

      const result = await controller.findAll();

      expect(mockGroupsService.findAll).toHaveBeenCalled();
      expect(mockGroupsService.findByProjectId).not.toHaveBeenCalled();
      expect(result).toEqual([mockGroup]);
    });

    it('should return groups for a specific project when projectId is provided', async () => {
      const projectId = 'project1';
      mockGroupsService.findByProjectId.mockResolvedValue([mockGroup]);

      const result = await controller.findAll(projectId);

      expect(mockGroupsService.findByProjectId).toHaveBeenCalledWith(projectId);
      expect(mockGroupsService.findAll).not.toHaveBeenCalled();
      expect(result).toEqual([mockGroup]);
    });
  });

  describe('findOne', () => {
    it('should return a group by id', async () => {
      const id = 'group1';
      mockGroupsService.findById.mockResolvedValue(mockGroup);

      const result = await controller.findOne(id);

      expect(mockGroupsService.findById).toHaveBeenCalledWith(id);
      expect(result).toEqual(mockGroup);
    });
  });

  describe('update', () => {
    it('should update a group', async () => {
      const id = 'group1';
      const updateGroupDto: UpdateGroupDto = {
        name: 'Updated Group',
      };

      mockGroupsService.update.mockResolvedValue({
        ...mockGroup,
        name: 'Updated Group',
      });

      const result = await controller.update(id, updateGroupDto);

      expect(mockGroupsService.update).toHaveBeenCalledWith(id, updateGroupDto);
      expect(result).toEqual({
        ...mockGroup,
        name: 'Updated Group',
      });
    });
  });

  describe('remove', () => {
    it('should remove a group', async () => {
      const id = 'group1';
      mockGroupsService.remove.mockResolvedValue(mockGroup);

      const result = await controller.remove(id);

      expect(mockGroupsService.remove).toHaveBeenCalledWith(id);
      expect(result).toEqual(mockGroup);
    });
  });

  describe('addPersonToGroup', () => {
    it('should add a person to a group', async () => {
      const groupId = 'group1';
      const personId = 'person1';
      const mockRelation = { groupId, personId };

      mockGroupsService.addPersonToGroup.mockResolvedValue(mockRelation);

      const result = await controller.addPersonToGroup(groupId, personId);

      expect(mockGroupsService.addPersonToGroup).toHaveBeenCalledWith(groupId, personId);
      expect(result).toEqual(mockRelation);
    });
  });

  describe('removePersonFromGroup', () => {
    it('should remove a person from a group', async () => {
      const groupId = 'group1';
      const personId = 'person1';
      const mockRelation = { groupId, personId };

      mockGroupsService.removePersonFromGroup.mockResolvedValue(mockRelation);

      const result = await controller.removePersonFromGroup(groupId, personId);

      expect(mockGroupsService.removePersonFromGroup).toHaveBeenCalledWith(groupId, personId);
      expect(result).toEqual(mockRelation);
    });
  });

  describe('getPersonsInGroup', () => {
    it('should get all persons in a group', async () => {
      const groupId = 'group1';
      const mockPersons = [{ person: mockPerson }];

      mockGroupsService.getPersonsInGroup.mockResolvedValue(mockPersons);

      const result = await controller.getPersonsInGroup(groupId);

      expect(mockGroupsService.getPersonsInGroup).toHaveBeenCalledWith(groupId);
      expect(result).toEqual(mockPersons);
    });
  });
});