import { Test, TestingModule } from '@nestjs/testing';
import { PersonsService } from './persons.service';
import { NotFoundException } from '@nestjs/common';
import { DRIZZLE } from '../../../database/database.module';
import { Gender, OralEaseLevel } from '../dto/create-person.dto';

describe('PersonsService', () => {
  let service: PersonsService;
  let mockDb: any;

  // Mock data
  const mockPerson = {
    id: 'person1',
    firstName: 'John',
    lastName: 'Doe',
    gender: Gender.MALE,
    technicalLevel: 3,
    hasTechnicalTraining: true,
    frenchSpeakingLevel: 4,
    oralEaseLevel: OralEaseLevel.COMFORTABLE,
    age: 30,
    projectId: 'project1',
    attributes: {},
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockGroup = {
    id: 'group1',
    name: 'Test Group',
    projectId: 'project1',
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockPersonToGroup = {
    personId: 'person1',
    groupId: 'group1',
  };

  // Mock database operations
  const mockDbOperations = {
    select: jest.fn().mockReturnThis(),
    from: jest.fn().mockReturnThis(),
    where: jest.fn().mockReturnThis(),
    insert: jest.fn().mockReturnThis(),
    values: jest.fn().mockReturnThis(),
    update: jest.fn().mockReturnThis(),
    set: jest.fn().mockReturnThis(),
    delete: jest.fn().mockReturnThis(),
    innerJoin: jest.fn().mockReturnThis(),
    returning: jest.fn().mockImplementation(() => {
      return [mockPerson];
    }),
  };

  beforeEach(async () => {
    mockDb = {
      ...mockDbOperations,
    };

    const module: TestingModule = await Test.createTestingModule({
      providers: [
        PersonsService,
        {
          provide: DRIZZLE,
          useValue: mockDb,
        },
      ],
    }).compile();

    service = module.get<PersonsService>(PersonsService);
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('create', () => {
    it('should create a new person', async () => {
      const createPersonDto = {
        firstName: 'John',
        lastName: 'Doe',
        gender: Gender.MALE,
        technicalLevel: 3,
        hasTechnicalTraining: true,
        frenchSpeakingLevel: 4,
        oralEaseLevel: OralEaseLevel.COMFORTABLE,
        projectId: 'project1',
      };

      const result = await service.create(createPersonDto);

      expect(mockDb.insert).toHaveBeenCalled();
      expect(mockDb.values).toHaveBeenCalledWith(createPersonDto);
      expect(result).toEqual(mockPerson);
    });
  });

  describe('findAll', () => {
    it('should return all persons', async () => {
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => [mockPerson]);

      const result = await service.findAll();

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(result).toEqual([mockPerson]);
    });
  });

  describe('findByProjectId', () => {
    it('should return persons for a specific project', async () => {
      const projectId = 'project1';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockPerson]);

      const result = await service.findByProjectId(projectId);

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual([mockPerson]);
    });
  });

  describe('findById', () => {
    it('should return a person by ID', async () => {
      const id = 'person1';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockPerson]);

      const result = await service.findById(id);

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockPerson);
    });

    it('should throw NotFoundException if person not found', async () => {
      const id = 'nonexistent';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => []);

      await expect(service.findById(id)).rejects.toThrow(NotFoundException);
    });
  });

  describe('update', () => {
    it('should update a person', async () => {
      const id = 'person1';
      const updatePersonDto = {
        firstName: 'Jane',
      };

      const result = await service.update(id, updatePersonDto);

      expect(mockDb.update).toHaveBeenCalled();
      expect(mockDb.set).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockPerson);
    });

    it('should throw NotFoundException if person not found', async () => {
      const id = 'nonexistent';
      const updatePersonDto = {
        firstName: 'Jane',
      };

      mockDb.update.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.set.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => []);

      await expect(service.update(id, updatePersonDto)).rejects.toThrow(NotFoundException);
    });
  });

  describe('remove', () => {
    it('should delete a person', async () => {
      const id = 'person1';

      const result = await service.remove(id);

      expect(mockDb.delete).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockPerson);
    });

    it('should throw NotFoundException if person not found', async () => {
      const id = 'nonexistent';

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => []);

      await expect(service.remove(id)).rejects.toThrow(NotFoundException);
    });
  });

  describe('findByProjectIdAndGroupId', () => {
    it('should return persons by project ID and group ID', async () => {
      const projectId = 'project1';
      const groupId = 'group1';

      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.innerJoin.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [{ person: mockPerson }]);

      const result = await service.findByProjectIdAndGroupId(projectId, groupId);

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.innerJoin).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual([{ person: mockPerson }]);
    });
  });

  describe('addToGroup', () => {
    it('should add a person to a group', async () => {
      const personId = 'person1';
      const groupId = 'group1';

      mockDb.insert.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.values.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [mockPersonToGroup]);

      const result = await service.addToGroup(personId, groupId);

      expect(mockDb.insert).toHaveBeenCalled();
      expect(mockDb.values).toHaveBeenCalledWith({
        personId,
        groupId,
      });
      expect(result).toEqual(mockPersonToGroup);
    });
  });

  describe('removeFromGroup', () => {
    it('should remove a person from a group', async () => {
      const personId = 'person1';
      const groupId = 'group1';

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [mockPersonToGroup]);

      const result = await service.removeFromGroup(personId, groupId);

      expect(mockDb.delete).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockPersonToGroup);
    });

    it('should throw NotFoundException if relation not found', async () => {
      const personId = 'nonexistent';
      const groupId = 'group1';

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => []);

      await expect(service.removeFromGroup(personId, groupId)).rejects.toThrow(NotFoundException);
    });
  });
});