import { Injectable, NotFoundException, Inject } from '@nestjs/common';
import { eq, and } from 'drizzle-orm';
import { DRIZZLE } from '../../../database/database.module';
import * as schema from '../../../database/schema';
import { CreatePersonDto } from '../dto/create-person.dto';
import { UpdatePersonDto } from '../dto/update-person.dto';

@Injectable()
export class PersonsService {
  constructor(@Inject(DRIZZLE) private readonly db: any) {}

  /**
   * Create a new person
   */
  async create(createPersonDto: CreatePersonDto) {
    const [person] = await this.db
      .insert(schema.persons)
      .values(createPersonDto)
      .returning();
    return person;
  }

  /**
   * Find all persons
   */
  async findAll() {
    return this.db.select().from(schema.persons);
  }

  /**
   * Find persons by project ID
   */
  async findByProjectId(projectId: string) {
    return this.db
      .select()
      .from(schema.persons)
      .where(eq(schema.persons.projectId, projectId));
  }

  /**
   * Find a person by ID
   */
  async findById(id: string) {
    const [person] = await this.db
      .select()
      .from(schema.persons)
      .where(eq(schema.persons.id, id));
    
    if (!person) {
      throw new NotFoundException(`Person with ID ${id} not found`);
    }
    
    return person;
  }

  /**
   * Update a person
   */
  async update(id: string, updatePersonDto: UpdatePersonDto) {
    const [person] = await this.db
      .update(schema.persons)
      .set({
        ...updatePersonDto,
        updatedAt: new Date(),
      })
      .where(eq(schema.persons.id, id))
      .returning();
    
    if (!person) {
      throw new NotFoundException(`Person with ID ${id} not found`);
    }
    
    return person;
  }

  /**
   * Delete a person
   */
  async remove(id: string) {
    const [person] = await this.db
      .delete(schema.persons)
      .where(eq(schema.persons.id, id))
      .returning();
    
    if (!person) {
      throw new NotFoundException(`Person with ID ${id} not found`);
    }
    
    return person;
  }

  /**
   * Find persons by project ID and group ID
   */
  async findByProjectIdAndGroupId(projectId: string, groupId: string) {
    return this.db
      .select({
        person: schema.persons,
      })
      .from(schema.persons)
      .innerJoin(
        schema.personToGroup,
        and(
          eq(schema.persons.id, schema.personToGroup.personId),
          eq(schema.personToGroup.groupId, groupId)
        )
      )
      .where(eq(schema.persons.projectId, projectId));
  }

  /**
   * Add a person to a group
   */
  async addToGroup(personId: string, groupId: string) {
    const [relation] = await this.db
      .insert(schema.personToGroup)
      .values({
        personId,
        groupId,
      })
      .returning();
    return relation;
  }

  /**
   * Remove a person from a group
   */
  async removeFromGroup(personId: string, groupId: string) {
    const [relation] = await this.db
      .delete(schema.personToGroup)
      .where(
        and(
          eq(schema.personToGroup.personId, personId),
          eq(schema.personToGroup.groupId, groupId)
        )
      )
      .returning();
    
    if (!relation) {
      throw new NotFoundException(`Person with ID ${personId} not found in group with ID ${groupId}`);
    }
    
    return relation;
  }
}